#include "PHASIC++/Channels/Single_Channel.H"
#include "ATOOLS/Org/Run_Parameter.H"
#include "PHASIC++/Channels/Channel_Elements.H"
#include "PHASIC++/Channels/Vegas.H"

using namespace PHASIC;
using namespace ATOOLS;

namespace PHASIC {
  class C2_0 : public Single_Channel {
    Info_Key m_kI_2_3,m_kZR24_80;
    Vegas* p_vegas;
  public:
    C2_0(int,int,Flavour*,Integration_Info * const);
    ~C2_0();
    void   GenerateWeight(Vec4D *,Cut_Data *);
    void   GeneratePoint(Vec4D *,Cut_Data *,double *);
    void   AddPoint(double);
    void   MPISync()                 { p_vegas->MPISync(); }
    void   Optimize()                { p_vegas->Optimize(); } 
    void   EndOptimize()             { p_vegas->EndOptimize(); } 
    void   WriteOut(std::string pId) { p_vegas->WriteOut(pId); } 
    void   ReadIn(std::string pId)   { p_vegas->ReadIn(pId); } 
    void   ISRInfo(int &,double &,double &);
    std::string ChID();
  };
}

extern "C" Single_Channel * Getter_C2_0(int nin,int nout,Flavour* fl,Integration_Info * const info) {
  return new C2_0(nin,nout,fl,info);
}

void C2_0::GeneratePoint(Vec4D * p,Cut_Data * cuts,double * _ran)
{
  double *ran = p_vegas->GeneratePoint(_ran);
  for(int i=0;i<rannum;i++) rans[i]=ran[i];
  Vec4D p23=p[0]+p[1];
  double s23_max = p23.Abs2();
  double s2 = ms[2];
  double s3 = ms[3];
  CE.Isotropic2Momenta(p23,s2,s3,p[2],p[3],ran[0],ran[1]);
}

void C2_0::GenerateWeight(Vec4D* p,Cut_Data * cuts)
{
  double wt = 1.;
  Vec4D p23=p[0]+p[1];
  double s23_max = p23.Abs2();
  double s2 = ms[2];
  double s3 = ms[3];
  if (m_kI_2_3.Weight()==ATOOLS::UNDEFINED_WEIGHT)
    m_kI_2_3<<CE.Isotropic2Weight(p[2],p[3],m_kI_2_3[0],m_kI_2_3[1]);
  wt *= m_kI_2_3.Weight();

  rans[0]= m_kI_2_3[0];
  rans[1]= m_kI_2_3[1];
  double vw = p_vegas->GenerateWeight(rans);
  if (wt!=0.) wt = vw/wt/pow(2.*M_PI,2*3.-4.);

  weight = wt;
}

C2_0::C2_0(int nin,int nout,Flavour* fl,Integration_Info * const info)
       : Single_Channel(nin,nout,fl)
{
  name = std::string("C2_0");
  rannum = 2;
  rans  = new double[rannum];
  m_kI_2_3.Assign(std::string("I_2_3"),2,0,info);
  m_kZR24_80.Assign(std::string("ZR24_80"),2,0,info);
  p_vegas = new Vegas(rannum,100,name);
}

C2_0::~C2_0()
{
  delete p_vegas;
}

void C2_0::ISRInfo(int & type,double & mass,double & width)
{
  type  = 1;
  mass  = Flavour((kf_code)(24)).Mass();
  width = Flavour((kf_code)(24)).Width();
}

void C2_0::AddPoint(double Value)
{
  Single_Channel::AddPoint(Value);
  p_vegas->AddPoint(Value,rans);
}
std::string C2_0::ChID()
{
  return std::string("CG2$I_2_3$ZR24_80$");
}
